@@ -13,27 +13,26 @@ import psycopg2.pool
 
 from swh.core.db import BaseDb
 from swh.core.db.common import db_transaction
-from swh.model import hashutil
+from swh.model.identifiers import CoreSWHID
 from swh.scheduler import get_scheduler
 from swh.scheduler.utils import create_oneshot_task_dict
 from swh.storage import get_storage
 from swh.vault.cache import VaultCache
 from swh.vault.cookers import COOKER_TYPES, get_cooker_cls
 from swh.vault.exc import NotFoundExc
-from swh.vault.interface import ObjectId
 
 cooking_task_name = "swh.vault.cooking_tasks.SWHCookingTask"
 
 NOTIF_EMAIL_FROM = '"Software Heritage Vault" ' "<bot@softwareheritage.org>"
-NOTIF_EMAIL_SUBJECT_SUCCESS = "Bundle ready: {obj_type} {short_id}"
-NOTIF_EMAIL_SUBJECT_FAILURE = "Bundle failed: {obj_type} {short_id}"
+NOTIF_EMAIL_SUBJECT_SUCCESS = "Bundle ready: {bundle_type} {short_id}"
+NOTIF_EMAIL_SUBJECT_FAILURE = "Bundle failed: {bundle_type} {short_id}"
 
 NOTIF_EMAIL_BODY_SUCCESS = """
 You have requested the following bundle from the Software Heritage
 Vault:
 
-Object Type: {obj_type}
-Object ID: {hex_id}
+Bundle Type: {bundle_type}
+Object SWHID: {swhid}
 
 This bundle is now available for download at the following address:
 
@@ -50,8 +49,8 @@ NOTIF_EMAIL_BODY_FAILURE = """
 You have requested the following bundle from the Software Heritage
 Vault:
 
-Object Type: {obj_type}
-Object ID: {hex_id}
+Bundle Type: {bundle_type}
+Object SWHID: {swhid}
 
 This bundle could not be cooked for the following reason:
 
@@ -64,10 +63,6 @@ The Software Heritage Developers
 """
 
 
-def batch_to_bytes(batch: List[Tuple[str, str]]) -> List[Tuple[str, bytes]]:
-    return [(obj_type, hashutil.hash_to_bytes(hex_id)) for obj_type, hex_id in batch]
-
-
 class VaultBackend:
     """
     Backend for the Software Heritage Vault.
@@ -98,138 +93,128 @@ class VaultBackend:
         if db is not self._db:
             db.put_conn()
 
-    def _compute_ids(self, obj_id: ObjectId) -> Tuple[str, bytes]:
-        """Internal method to reconcile multiple possible inputs
-
-        """
-        if isinstance(obj_id, str):
-            return obj_id, hashutil.hash_to_bytes(obj_id)
-        return hashutil.hash_to_hex(obj_id), obj_id
-
     @db_transaction()
     def progress(
         self,
-        obj_type: str,
-        obj_id: ObjectId,
+        bundle_type: str,
+        swhid: CoreSWHID,
         raise_notfound: bool = True,
         db=None,
         cur=None,
     ) -> Optional[Dict[str, Any]]:
-        hex_id, obj_id = self._compute_ids(obj_id)
         cur.execute(
             """
-            SELECT id, type, object_id, task_id, task_status, sticky,
+            SELECT id, type, swhid, task_id, task_status, sticky,
                    ts_created, ts_done, ts_last_access, progress_msg
             FROM vault_bundle
-            WHERE type = %s AND object_id = %s""",
-            (obj_type, obj_id),
+            WHERE type = %s AND swhid = %s""",
+            (bundle_type, str(swhid)),
         )
         res = cur.fetchone()
         if not res:
             if raise_notfound:
-                raise NotFoundExc(f"{obj_type} {hex_id} was not found.")
+                raise NotFoundExc(f"{bundle_type} {swhid} was not found.")
             return None
-
-        res["object_id"] = hashutil.hash_to_hex(res["object_id"])
+        res["swhid"] = CoreSWHID.from_string(res["swhid"])
         return res
 
-    def _send_task(self, obj_type: str, hex_id: ObjectId):
+    def _send_task(self, bundle_type: str, swhid: CoreSWHID):
         """Send a cooking task to the celery scheduler"""
-        task = create_oneshot_task_dict("cook-vault-bundle", obj_type, hex_id)
+        task = create_oneshot_task_dict("cook-vault-bundle", bundle_type, str(swhid))
         added_tasks = self.scheduler.create_tasks([task])
         return added_tasks[0]["id"]
 
     @db_transaction()
     def create_task(
-        self, obj_type: str, obj_id: bytes, sticky: bool = False, db=None, cur=None
+        self,
+        bundle_type: str,
+        swhid: CoreSWHID,
+        sticky: bool = False,
+        db=None,
+        cur=None,
     ):
         """Create and send a cooking task"""
-        hex_id, obj_id = self._compute_ids(obj_id)
-
-        cooker_class = get_cooker_cls(obj_type)
-        cooker = cooker_class(obj_type, hex_id, backend=self, storage=self.storage)
+        cooker_class = get_cooker_cls(bundle_type, swhid.object_type)
+        cooker = cooker_class(swhid, backend=self, storage=self.storage)
 
         if not cooker.check_exists():
-            raise NotFoundExc(f"{obj_type} {hex_id} was not found.")
+            raise NotFoundExc(f"{bundle_type} {swhid} was not found.")
 
         cur.execute(
             """
-            INSERT INTO vault_bundle (type, object_id, sticky)
+            INSERT INTO vault_bundle (type, swhid, sticky)
             VALUES (%s, %s, %s)""",
-            (obj_type, obj_id, sticky),
+            (bundle_type, str(swhid), sticky),
         )
         db.conn.commit()
 
-        task_id = self._send_task(obj_type, hex_id)
+        task_id = self._send_task(bundle_type, swhid)
 
         cur.execute(
             """
             UPDATE vault_bundle
             SET task_id = %s
-            WHERE type = %s AND object_id = %s""",
-            (task_id, obj_type, obj_id),
+            WHERE type = %s AND swhid = %s""",
+            (task_id, bundle_type, str(swhid)),
         )
 
     @db_transaction()
     def add_notif_email(
-        self, obj_type: str, obj_id: bytes, email: str, db=None, cur=None
+        self, bundle_type: str, swhid: CoreSWHID, email: str, db=None, cur=None
     ):
         """Add an e-mail address to notify when a given bundle is ready"""
         cur.execute(
             """
             INSERT INTO vault_notif_email (email, bundle_id)
             VALUES (%s, (SELECT id FROM vault_bundle
-                         WHERE type = %s AND object_id = %s))""",
-            (email, obj_type, obj_id),
+                         WHERE type = %s AND swhid = %s))""",
+            (email, bundle_type, str(swhid)),
         )
 
-    def put_bundle(self, obj_type: str, obj_id: ObjectId, bundle) -> bool:
-        _, obj_id = self._compute_ids(obj_id)
-        self.cache.add(obj_type, obj_id, bundle)
+    def put_bundle(self, bundle_type: str, swhid: CoreSWHID, bundle) -> bool:
+        self.cache.add(bundle_type, swhid, bundle)
         return True
 
     @db_transaction()
     def cook(
         self,
-        obj_type: str,
-        obj_id: ObjectId,
+        bundle_type: str,
+        swhid: CoreSWHID,
         *,
         sticky: bool = False,
         email: Optional[str] = None,
         db=None,
         cur=None,
     ) -> Dict[str, Any]:
-        hex_id, obj_id = self._compute_ids(obj_id)
-        info = self.progress(obj_type, obj_id, raise_notfound=False)
+        info = self.progress(bundle_type, swhid, raise_notfound=False)
 
-        if obj_type not in COOKER_TYPES:
-            raise NotFoundExc(f"{obj_type} is an unknown type.")
+        if bundle_type not in COOKER_TYPES:
+            raise NotFoundExc(f"{bundle_type} is an unknown type.")
 
         # If there's a failed bundle entry, delete it first.
         if info is not None and info["task_status"] == "failed":
-            obj_id = hashutil.hash_to_bytes(obj_id)
             cur.execute(
-                "DELETE FROM vault_bundle WHERE type = %s AND object_id = %s",
-                (obj_type, obj_id),
+                "DELETE FROM vault_bundle WHERE type = %s AND swhid = %s",
+                (bundle_type, str(swhid)),
             )
             db.conn.commit()
             info = None
 
         # If there's no bundle entry, create the task.
         if info is None:
-            self.create_task(obj_type, obj_id, sticky)
+            self.create_task(bundle_type, swhid, sticky)
 
         if email is not None:
             # If the task is already done, send the email directly
             if info is not None and info["task_status"] == "done":
                 self.send_notification(
-                    None, email, obj_type, hex_id, info["task_status"]
+                    None, email, bundle_type, swhid, info["task_status"]
                 )
             # Else, add it to the notification queue
             else:
-                self.add_notif_email(obj_type, obj_id, email)
+                self.add_notif_email(bundle_type, swhid, email)
 
-        return self.progress(obj_type, obj_id)
+        return self.progress(bundle_type, swhid)
 
     @db_transaction()
     def batch_cook(
@@ -239,9 +224,9 @@ class VaultBackend:
         # psycopg2 >= 2.7 (only available on postgresql servers)
         from psycopg2.extras import execute_values
 
-        for obj_type, _ in batch:
-            if obj_type not in COOKER_TYPES:
-                raise NotFoundExc(f"{obj_type} is an unknown type.")
+        for bundle_type, _ in batch:
+            if bundle_type not in COOKER_TYPES:
+                raise NotFoundExc(f"{bundle_type} is an unknown type.")
 
         cur.execute(
             """
@@ -250,32 +235,31 @@ class VaultBackend:
             RETURNING id"""
         )
         batch_id = cur.fetchone()["id"]
-        batch_bytes = batch_to_bytes(batch)
 
         # Delete all failed bundles from the batch
         cur.execute(
             """
             DELETE FROM vault_bundle
             WHERE task_status = 'failed'
-              AND (type, object_id) IN %s""",
-            (tuple(batch_bytes),),
+              AND (type, swhid) IN %s""",
+            (tuple(batch),),
         )
 
         # Insert all the bundles, return the new ones
         execute_values(
             cur,
             """
-            INSERT INTO vault_bundle (type, object_id)
+            INSERT INTO vault_bundle (type, swhid)
             VALUES %s ON CONFLICT DO NOTHING""",
-            batch_bytes,
+            batch,
         )
 
         # Get the bundle ids and task status
         cur.execute(
             """
-            SELECT id, type, object_id, task_id FROM vault_bundle
-            WHERE (type, object_id) IN %s""",
-            (tuple(batch_bytes),),
+            SELECT id, type, swhid, task_id FROM vault_bundle
+            WHERE (type, swhid) IN %s""",
+            (tuple(batch),),
         )
         bundles = cur.fetchall()
 
@@ -292,15 +276,13 @@ class VaultBackend:
 
         # Get the tasks to fetch
         batch_new = [
-            (row["type"], bytes(row["object_id"]))
+            (row["type"], CoreSWHID.from_string(row["swhid"]))
             for row in bundles
             if row["task_id"] is None
         ]
 
         # Send the tasks
-        args_batch = [
-            (obj_type, hashutil.hash_to_hex(obj_id)) for obj_type, obj_id in batch_new
-        ]
+        args_batch = [(bundle_type, swhid) for bundle_type, swhid in batch_new]
         # TODO: change once the scheduler handles priority tasks
         tasks = [
             create_oneshot_task_dict("swh-vault-batch-cooking", *args)
@@ -309,8 +291,8 @@ class VaultBackend:
 
         added_tasks = self.scheduler.create_tasks(tasks)
         tasks_ids_bundle_ids = [
-            (task_id, obj_type, obj_id)
-            for task_id, (obj_type, obj_id) in zip(
+            (task_id, bundle_type, swhid)
+            for task_id, (bundle_type, swhid) in zip(
                 [task["id"] for task in added_tasks], batch_new
             )
         ]
@@ -321,8 +303,8 @@ class VaultBackend:
             """
             UPDATE vault_bundle
             SET task_id = s_task_id
-            FROM (VALUES %s) AS sub (s_task_id, s_type, s_object_id)
-            WHERE type = s_type::cook_type AND object_id = s_object_id """,
+            FROM (VALUES %s) AS sub (s_task_id, s_type, s_swhid)
+            WHERE type = s_type::cook_type AND swhid = s_swhid """,
             tasks_ids_bundle_ids,
         )
         return {"id": batch_id}
@@ -332,7 +314,7 @@ class VaultBackend:
         cur.execute(
             """
             SELECT vault_bundle.id as id,
-                   type, object_id, task_id, task_status, sticky,
+                   type, swhid, task_id, task_status, sticky,
                    ts_created, ts_done, ts_last_access, progress_msg
             FROM vault_batch_bundle
             LEFT JOIN vault_bundle ON vault_bundle.id = bundle_id
@@ -344,7 +326,7 @@ class VaultBackend:
             raise NotFoundExc(f"Batch {batch_id} does not exist.")
 
         for bundle in bundles:
-            bundle["object_id"] = hashutil.hash_to_hex(bundle["object_id"])
+            bundle["swhid"] = CoreSWHID.from_string(bundle["swhid"])
 
         counter = collections.Counter(b["status"] for b in bundles)
         res = {
@@ -357,87 +339,82 @@ class VaultBackend:
         return res
 
     @db_transaction()
-    def is_available(self, obj_type: str, obj_id: ObjectId, db=None, cur=None):
+    def is_available(self, bundle_type: str, swhid: CoreSWHID, db=None, cur=None):
         """Check whether a bundle is available for retrieval"""
-        info = self.progress(obj_type, obj_id, raise_notfound=False, cur=cur)
-        obj_id = hashutil.hash_to_bytes(obj_id)
+        info = self.progress(bundle_type, swhid, raise_notfound=False, cur=cur)
         return (
             info is not None
             and info["task_status"] == "done"
-            and self.cache.is_cached(obj_type, obj_id)
+            and self.cache.is_cached(bundle_type, swhid)
         )
 
     @db_transaction()
     def fetch(
-        self, obj_type: str, obj_id: ObjectId, raise_notfound=True, db=None, cur=None
+        self, bundle_type: str, swhid: CoreSWHID, raise_notfound=True, db=None, cur=None
     ) -> Optional[bytes]:
         """Retrieve a bundle from the cache"""
-        hex_id, obj_id = self._compute_ids(obj_id)
-        available = self.is_available(obj_type, obj_id, cur=cur)
+        available = self.is_available(bundle_type, swhid, cur=cur)
         if not available:
             if raise_notfound:
-                raise NotFoundExc(f"{obj_type} {hex_id} is not available.")
+                raise NotFoundExc(f"{bundle_type} {swhid} is not available.")
             return None
-        self.update_access_ts(obj_type, obj_id, cur=cur)
-        return self.cache.get(obj_type, obj_id)
+        self.update_access_ts(bundle_type, swhid, cur=cur)
+        return self.cache.get(bundle_type, swhid)
 
     @db_transaction()
-    def update_access_ts(self, obj_type: str, obj_id: bytes, db=None, cur=None):
+    def update_access_ts(self, bundle_type: str, swhid: CoreSWHID, db=None, cur=None):
         """Update the last access timestamp of a bundle"""
         cur.execute(
             """
             UPDATE vault_bundle
             SET ts_last_access = NOW()
-            WHERE type = %s AND object_id = %s""",
-            (obj_type, obj_id),
+            WHERE type = %s AND swhid = %s""",
+            (bundle_type, str(swhid)),
         )
 
     @db_transaction()
     def set_status(
-        self, obj_type: str, obj_id: ObjectId, status: str, db=None, cur=None
+        self, bundle_type: str, swhid: CoreSWHID, status: str, db=None, cur=None
     ) -> bool:
-        obj_id = hashutil.hash_to_bytes(obj_id)
         req = (
             """
                UPDATE vault_bundle
                SET task_status = %s """
             + (""", ts_done = NOW() """ if status == "done" else "")
-            + """WHERE type = %s AND object_id = %s"""
+            + """WHERE type = %s AND swhid = %s"""
         )
-        cur.execute(req, (status, obj_type, obj_id))
+        cur.execute(req, (status, bundle_type, str(swhid)))
         return True
 
     @db_transaction()
     def set_progress(
-        self, obj_type: str, obj_id: ObjectId, progress: str, db=None, cur=None
+        self, bundle_type: str, swhid: CoreSWHID, progress: str, db=None, cur=None
     ) -> bool:
-        obj_id = hashutil.hash_to_bytes(obj_id)
         cur.execute(
             """
             UPDATE vault_bundle
             SET progress_msg = %s
-            WHERE type = %s AND object_id = %s""",
-            (progress, obj_type, obj_id),
+            WHERE type = %s AND swhid = %s""",
+            (progress, bundle_type, str(swhid)),
         )
         return True
 
     @db_transaction()
-    def send_notif(self, obj_type: str, obj_id: ObjectId, db=None, cur=None) -> bool:
-        hex_id, obj_id = self._compute_ids(obj_id)
+    def send_notif(self, bundle_type: str, swhid: CoreSWHID, db=None, cur=None) -> bool:
         cur.execute(
             """
             SELECT vault_notif_email.id AS id, email, task_status, progress_msg
             FROM vault_notif_email
             INNER JOIN vault_bundle ON bundle_id = vault_bundle.id
-            WHERE vault_bundle.type = %s AND vault_bundle.object_id = %s""",
-            (obj_type, obj_id),
+            WHERE vault_bundle.type = %s AND vault_bundle.swhid = %s""",
+            (bundle_type, str(swhid)),
         )
         for d in cur:
             self.send_notification(
                 d["id"],
                 d["email"],
-                obj_type,
-                hex_id,
+                bundle_type,
+                swhid,
                 status=d["task_status"],
                 progress_msg=d["progress_msg"],
             )
@@ -448,15 +425,15 @@ class VaultBackend:
         self,
         n_id: Optional[int],
         email: str,
-        obj_type: str,
-        hex_id: str,
+        bundle_type: str,
+        swhid: CoreSWHID,
         status: str,
         progress_msg: Optional[str] = None,
         db=None,
         cur=None,
     ) -> None:
         """Send the notification of a bundle to a specific e-mail"""
-        short_id = hex_id[:7]
+        short_id = swhid.object_id.hex()[:7]
 
         # TODO: instead of hardcoding this, we should probably:
         # * add a "fetch_url" field in the vault_notif_email table
@@ -465,24 +442,24 @@ class VaultBackend:
         #   the table
         # * use this url for the notification e-mail
         url = "https://archive.softwareheritage.org/api/1/vault/{}/{}/" "raw".format(
-            obj_type, hex_id
+            bundle_type, swhid
         )
 
         if status == "done":
             text = NOTIF_EMAIL_BODY_SUCCESS.strip()
-            text = text.format(obj_type=obj_type, hex_id=hex_id, url=url)
+            text = text.format(bundle_type=bundle_type, swhid=swhid, url=url)
             msg = MIMEText(text)
             msg["Subject"] = NOTIF_EMAIL_SUBJECT_SUCCESS.format(
-                obj_type=obj_type, short_id=short_id
+                bundle_type=bundle_type, short_id=short_id
             )
         elif status == "failed":
             text = NOTIF_EMAIL_BODY_FAILURE.strip()
             text = text.format(
-                obj_type=obj_type, hex_id=hex_id, progress_msg=progress_msg
+                bundle_type=bundle_type, swhid=swhid, progress_msg=progress_msg
             )
             msg = MIMEText(text)
             msg["Subject"] = NOTIF_EMAIL_SUBJECT_FAILURE.format(
-                obj_type=obj_type, short_id=short_id
+                bundle_type=bundle_type, short_id=short_id
             )
         else:
             raise RuntimeError(
@@ -527,7 +504,7 @@ class VaultBackend:
                 WHERE sticky = false
                 {}
             )
-            RETURNING type, object_id
+            RETURNING type, swhid
             """.format(
                 cond
             ),
@@ -535,7 +512,7 @@ class VaultBackend:
         )
 
         for d in cur:
-            self.cache.delete(d["type"], bytes(d["object_id"]))
+            self.cache.delete(d["type"], CoreSWHID.from_string(d["swhid"]))
 
     @db_transaction()
     def cache_expire_oldest(self, n=1, by="last_access", db=None, cur=None) -> None:
